<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{{ title }}</title>
</head>
<body>
    {{ content }}

    <script src="https://cdn.plot.ly/plotly-2.16.1.min.js"></script>

    {% for chart in charts %}
    <script src="../charts/{{ chart }}/chart.js"></script>
    {% endfor %}

</body>
</html>
